smallworld.exceptions¶
- exception smallworld.exceptions.Error¶
Common base class for all exceptions.
- add_note()¶
Exception.add_note(note) – add a note to the exception
- with_traceback()¶
Exception.with_traceback(tb) – set self.__traceback__ to tb and return self.
- exception smallworld.exceptions.ConfigurationError¶
Raised when there is a problem with configuration.
- add_note()¶
Exception.add_note(note) – add a note to the exception
- with_traceback()¶
Exception.with_traceback(tb) – set self.__traceback__ to tb and return self.
- exception smallworld.exceptions.EmulationError¶
Raised when emulation fails.
- add_note()¶
Exception.add_note(note) – add a note to the exception
- with_traceback()¶
Exception.with_traceback(tb) – set self.__traceback__ to tb and return self.
- exception smallworld.exceptions.EmulationStop¶
Base class for all emulation stopping exceptions.
- add_note()¶
Exception.add_note(note) – add a note to the exception
- with_traceback()¶
Exception.with_traceback(tb) – set self.__traceback__ to tb and return self.
- exception smallworld.exceptions.EmulationBounds¶
Raised when execution goes out of bounds.
- add_note()¶
Exception.add_note(note) – add a note to the exception
- with_traceback()¶
Exception.with_traceback(tb) – set self.__traceback__ to tb and return self.
- exception smallworld.exceptions.EmulationExitpoint¶
Raised when execution hits an exit point.
- add_note()¶
Exception.add_note(note) – add a note to the exception
- with_traceback()¶
Exception.with_traceback(tb) – set self.__traceback__ to tb and return self.
- exception smallworld.exceptions.EmulationException(exception: Exception)¶
Raised when the underlying emulator fails.
This wraps known exceptions thrown by an Emulator.
- Parameters:
exception – The original exception thrown.
- add_note()¶
Exception.add_note(note) – add a note to the exception
- with_traceback()¶
Exception.with_traceback(tb) – set self.__traceback__ to tb and return self.
- exception smallworld.exceptions.ImpreciseModelError¶
Raised when a non-whitecarded imprecise function model is called
- add_note()¶
Exception.add_note(note) – add a note to the exception
- with_traceback()¶
Exception.with_traceback(tb) – set self.__traceback__ to tb and return self.
- exception smallworld.exceptions.SymbolicValueError¶
Raised if you try to collapse a symbolic value to a concrete one
- add_note()¶
Exception.add_note(note) – add a note to the exception
- with_traceback()¶
Exception.with_traceback(tb) – set self.__traceback__ to tb and return self.
- exception smallworld.exceptions.UnsatError¶
Raised if a symbolic expression is unsatisfiable given constraints
- add_note()¶
Exception.add_note(note) – add a note to the exception
- with_traceback()¶
Exception.with_traceback(tb) – set self.__traceback__ to tb and return self.
- exception smallworld.exceptions.EmulationFailure(msg: str, pc: int, *args)¶
Raised if the code being emulated faults somehow
- add_note()¶
Exception.add_note(note) – add a note to the exception
- with_traceback()¶
Exception.with_traceback(tb) – set self.__traceback__ to tb and return self.
- exception smallworld.exceptions.EmulationExecFailure(msg: str, pc: int, *args)¶
Raised if the current instruction failed to execute
- add_note()¶
Exception.add_note(note) – add a note to the exception
- with_traceback()¶
Exception.with_traceback(tb) – set self.__traceback__ to tb and return self.
- exception smallworld.exceptions.EmulationExecInvalidFailure(msg: str, pc: int, instruction: Any | None, *args)¶
Raised if the current instruction is invalid for this platform
- add_note()¶
Exception.add_note(note) – add a note to the exception
- with_traceback()¶
Exception.with_traceback(tb) – set self.__traceback__ to tb and return self.
- exception smallworld.exceptions.EmulationExecExceptionFailure(msg: str, pc: int, *args)¶
Raised if the current instruction raised an unhandled trap, interrupt, or exception.
The way different emulators report this is inconsistent. The number reported is NOT a platform-specific trap number.
- add_note()¶
Exception.add_note(note) – add a note to the exception
- with_traceback()¶
Exception.with_traceback(tb) – set self.__traceback__ to tb and return self.
- exception smallworld.exceptions.EmulationMemoryFailure(msg: str, pc: int, *args, operands: List[Any] | None = None, address: int | None = None)¶
Raised if something went wrong accessing memory
- add_note()¶
Exception.add_note(note) – add a note to the exception
- with_traceback()¶
Exception.with_traceback(tb) – set self.__traceback__ to tb and return self.
- exception smallworld.exceptions.EmulationReadFailure(msg: str, pc: int, *args, operands: List[Any] | None = None, address: int | None = None)¶
Raised if something went wrong reading memory
- add_note()¶
Exception.add_note(note) – add a note to the exception
- with_traceback()¶
Exception.with_traceback(tb) – set self.__traceback__ to tb and return self.
- exception smallworld.exceptions.EmulationReadUnmappedFailure(msg: str, pc: int, *args, operands: List[Any] | None = None, address: int | None = None)¶
Raised if the current instruction tried to read unmapped memory
- add_note()¶
Exception.add_note(note) – add a note to the exception
- with_traceback()¶
Exception.with_traceback(tb) – set self.__traceback__ to tb and return self.
- exception smallworld.exceptions.EmulationReadProtectedFailure(msg: str, pc: int, *args, operands: List[Any] | None = None, address: int | None = None)¶
Raised if the current instruction tried to read read-protected memory
- add_note()¶
Exception.add_note(note) – add a note to the exception
- with_traceback()¶
Exception.with_traceback(tb) – set self.__traceback__ to tb and return self.
- exception smallworld.exceptions.EmulationReadUnalignedFailure(msg: str, pc: int, *args, operands: List[Any] | None = None, address: int | None = None)¶
Raised if the current instruction tried to read an unaligned address
- add_note()¶
Exception.add_note(note) – add a note to the exception
- with_traceback()¶
Exception.with_traceback(tb) – set self.__traceback__ to tb and return self.
- exception smallworld.exceptions.EmulationWriteFailure(msg: str, pc: int, *args, operands: List[Any] | None = None, address: int | None = None)¶
Raised if something went wrong writing memory
- add_note()¶
Exception.add_note(note) – add a note to the exception
- with_traceback()¶
Exception.with_traceback(tb) – set self.__traceback__ to tb and return self.
- exception smallworld.exceptions.EmulationWriteUnmappedFailure(msg: str, pc: int, *args, operands: List[Any] | None = None, address: int | None = None)¶
Raised if the current instruction tried to write to unmapped memory
- add_note()¶
Exception.add_note(note) – add a note to the exception
- with_traceback()¶
Exception.with_traceback(tb) – set self.__traceback__ to tb and return self.
- exception smallworld.exceptions.EmulationWriteProtectedFailure(msg: str, pc: int, *args, operands: List[Any] | None = None, address: int | None = None)¶
Raised if the current instruction tried to write to write-protected memory
- add_note()¶
Exception.add_note(note) – add a note to the exception
- with_traceback()¶
Exception.with_traceback(tb) – set self.__traceback__ to tb and return self.
- exception smallworld.exceptions.EmulationWriteUnalignedFailure(msg: str, pc: int, *args, operands: List[Any] | None = None, address: int | None = None)¶
Raised if the current instruction tried to write to an unaligned address
- add_note()¶
Exception.add_note(note) – add a note to the exception
- with_traceback()¶
Exception.with_traceback(tb) – set self.__traceback__ to tb and return self.
- exception smallworld.exceptions.EmulationFetchFailure(msg: str, pc: int, *args, operands: List[Any] | None = None, address: int | None = None)¶
Raised if something went wrong fetching an instruction
- add_note()¶
Exception.add_note(note) – add a note to the exception
- with_traceback()¶
Exception.with_traceback(tb) – set self.__traceback__ to tb and return self.
- exception smallworld.exceptions.EmulationFetchUnmappedFailure(msg: str, pc: int, *args, operands: List[Any] | None = None, address: int | None = None)¶
Raised if an instruction fetch hit unmapped memory
- add_note()¶
Exception.add_note(note) – add a note to the exception
- with_traceback()¶
Exception.with_traceback(tb) – set self.__traceback__ to tb and return self.
- exception smallworld.exceptions.EmulationFetchProtectedFailure(msg: str, pc: int, *args, operands: List[Any] | None = None, address: int | None = None)¶
Raised if an instruction fetch hit exec-protected memory
- add_note()¶
Exception.add_note(note) – add a note to the exception
- with_traceback()¶
Exception.with_traceback(tb) – set self.__traceback__ to tb and return self.
- exception smallworld.exceptions.EmulationFetchUnalignedFailure(msg: str, pc: int, *args, operands: List[Any] | None = None, address: int | None = None)¶
Raised if an instruction fetch was for an unaligned address
- add_note()¶
Exception.add_note(note) – add a note to the exception
- with_traceback()¶
Exception.with_traceback(tb) – set self.__traceback__ to tb and return self.
- exception smallworld.exceptions.UnsupportedModelError¶
Raised when a model of an unsupported function gets called
- add_note()¶
Exception.add_note(note) – add a note to the exception
- with_traceback()¶
Exception.with_traceback(tb) – set self.__traceback__ to tb and return self.